@@ -18,12 +18,12 @@ def fileformat(request):
 
 SIMPLE_RECORDS = {
     "fasta": [
-        dnaio.Sequence("first_sequence", "SEQUENCE1"),
-        dnaio.Sequence("second_sequence", "SEQUENCE2"),
+        dnaio.SequenceRecord("first_sequence", "SEQUENCE1"),
+        dnaio.SequenceRecord("second_sequence", "SEQUENCE2"),
     ],
     "fastq": [
-        dnaio.Sequence("first_sequence", "SEQUENCE1", ":6;;8<=:<"),
-        dnaio.Sequence("second_sequence", "SEQUENCE2", "83<??:(61"),
+        dnaio.SequenceRecord("first_sequence", "SEQUENCE1", ":6;;8<=:<"),
+        dnaio.SequenceRecord("second_sequence", "SEQUENCE2", "83<??:(61"),
     ],
 }
 
@@ -31,16 +31,21 @@ SIMPLE_RECORDS = {
 def formatted_sequence(record, fileformat):
     if fileformat == "fastq":
         return "@{}\n{}\n+\n{}\n".format(record.name, record.sequence, record.qualities)
+    elif fileformat == "fastq_bytes":
+        return b"@%b\n%b\n+\n%b\n" % (record.name, record.sequence, record.qualities)
     else:
         return ">{}\n{}\n".format(record.name, record.sequence)
 
 
 def formatted_sequences(records, fileformat):
-    return "".join(formatted_sequence(record, fileformat) for record in records)
+    record_iter = (formatted_sequence(record, fileformat) for record in records)
+    if fileformat == "fastq_bytes":
+        return b"".join(record_iter)
+    return "".join(record_iter)
 
 
 def test_formatted_sequence():
-    s = dnaio.Sequence("s1", "ACGT", "HHHH")
+    s = dnaio.SequenceRecord("s1", "ACGT", "HHHH")
     assert ">s1\nACGT\n" == formatted_sequence(s, "fasta")
     assert "@s1\nACGT\n+\nHHHH\n" == formatted_sequence(s, "fastq")
 
@@ -79,23 +84,33 @@ def test_read_pathlib_path(fileformat, extension):
 def test_read_opener(fileformat, extension):
     def my_opener(path, mode):
         import io
+
         if fileformat == "fasta":
             data = b">read\nACG\n"
         else:
             data = b"@read\nACG\n+\nHHH\n"
         return io.BytesIO(data)
 
-    with dnaio.open("totally-ignored-filename." + fileformat + extension, opener=my_opener) as f:
+    with dnaio.open(
+        "totally-ignored-filename." + fileformat + extension, opener=my_opener
+    ) as f:
         records = list(f)
     assert len(records) == 1
     assert records[0].name == "read"
     assert records[0].sequence == "ACG"
 
 
+def test_read_paired_fasta():
+    path = "tests/data/simple.fasta"
+    with dnaio.open(file1=path, file2=path) as f:
+        list(f)
+
+
 @pytest.mark.parametrize("interleaved", [False, True])
 def test_paired_opener(fileformat, extension, interleaved):
     def my_opener(_path, _mode):
         import io
+
         if fileformat == "fasta":
             data = b">read\nACG\n"
         else:
@@ -121,32 +136,31 @@ def test_paired_opener(fileformat, extension, interleaved):
 
 def test_detect_fastq_from_content():
     """FASTQ file that is not named .fastq"""
-    with dnaio.open('tests/data/missingextension') as f:
+    with dnaio.open("tests/data/missingextension") as f:
         record = next(iter(f))
-        assert record.name == 'prefix:1_13_573/1'
+        assert record.name == "prefix:1_13_573/1"
 
 
 def test_detect_compressed_fastq_from_content():
     """Compressed FASTQ file that is not named .fastq.gz"""
-    with dnaio.open('tests/data/missingextension.gz') as f:
+    with dnaio.open("tests/data/missingextension.gz") as f:
         record = next(iter(f))
-    assert record.name == 'prefix:1_13_573/1'
+    assert record.name == "prefix:1_13_573/1"
 
 
-def test_write(tmpdir, extension):
-    s = dnaio.Sequence('name', 'ACGT', 'HHHH')
-    out_fastq = tmpdir.join("out.fastq" + extension)
-    with dnaio.open(str(out_fastq), mode='w') as f:
-        f.write(s)
+def test_write(tmp_path, extension):
+    out_fastq = tmp_path / ("out.fastq" + extension)
+    with dnaio.open(str(out_fastq), mode="w") as f:
+        f.write(dnaio.SequenceRecord("name", "ACGT", "HHHH"))
     with xopen(out_fastq) as f:
-        assert f.read() == '@name\nACGT\n+\nHHHH\n'
+        assert f.read() == "@name\nACGT\n+\nHHHH\n"
 
 
-def test_write_with_xopen(tmpdir, fileformat, extension):
-    s = dnaio.Sequence('name', 'ACGT', 'HHHH')
-    out_fastq = str(tmpdir.join("out." + fileformat + extension))
-    with xopen(out_fastq, 'wb') as outer_f:
-        with dnaio.open(outer_f, mode='w', fileformat=fileformat) as f:
+def test_write_with_xopen(tmp_path, fileformat, extension):
+    s = dnaio.SequenceRecord("name", "ACGT", "HHHH")
+    out_fastq = tmp_path / ("out." + fileformat + extension)
+    with xopen(out_fastq, "wb") as outer_f:
+        with dnaio.open(outer_f, mode="w", fileformat=fileformat) as f:
             f.write(s)
 
     with xopen(out_fastq) as f:
@@ -156,9 +170,9 @@ def test_write_with_xopen(tmpdir, fileformat, extension):
             assert f.read() == "@name\nACGT\n+\nHHHH\n"
 
 
-def test_write_pathlib(tmpdir, fileformat, extension):
-    s1 = dnaio.Sequence("s1", "ACGT", "HHHH")
-    path = Path(str(tmpdir / ("out." + fileformat + extension)))
+def test_write_str_path(tmp_path, fileformat, extension):
+    s1 = dnaio.SequenceRecord("s1", "ACGT", "HHHH")
+    path = str(tmp_path / ("out." + fileformat + extension))
     with dnaio.open(path, mode="w") as f:
         f.write(s1)
     if fileformat == "fasta":
@@ -169,25 +183,25 @@ def test_write_pathlib(tmpdir, fileformat, extension):
         assert f.read() == expected
 
 
-def test_write_paired_same_path(tmpdir):
-    path1 = str(tmpdir / "same.fastq")
-    path2 = str(tmpdir / "same.fastq")
+def test_write_paired_same_path(tmp_path):
+    path1 = tmp_path / "same.fastq"
+    path2 = tmp_path / "same.fastq"
     with pytest.raises(ValueError):
         with dnaio.open(file1=path1, file2=path2, mode="w"):
             pass
 
 
-def test_write_paired(tmpdir, fileformat, extension):
+def test_write_paired(tmp_path, fileformat, extension):
     r1 = [
-        dnaio.Sequence("s1", "ACGT", "HHHH"),
-        dnaio.Sequence("s2", "CGCA", "8383"),
+        dnaio.SequenceRecord("s1", "ACGT", "HHHH"),
+        dnaio.SequenceRecord("s2", "CGCA", "8383"),
     ]
     r2 = [
-        dnaio.Sequence("t1", "TCGT", "5HHH"),
-        dnaio.Sequence("t2", "TGCA", "5383"),
+        dnaio.SequenceRecord("t1", "TCGT", "5HHH"),
+        dnaio.SequenceRecord("t2", "TGCA", "5383"),
     ]
-    path1 = str(tmpdir / ("out.1." + fileformat + extension))
-    path2 = str(tmpdir / ("out.2." + fileformat + extension))
+    path1 = tmp_path / ("out.1." + fileformat + extension)
+    path2 = tmp_path / ("out.2." + fileformat + extension)
 
     with dnaio.open(path1, file2=path2, fileformat=fileformat, mode="w") as f:
         f.write(r1[0], r2[0])
@@ -198,16 +212,16 @@ def test_write_paired(tmpdir, fileformat, extension):
         assert formatted_sequences(r2, fileformat) == f.read()
 
 
-def test_write_interleaved(tmpdir, fileformat, extension):
+def test_write_interleaved(tmp_path, fileformat, extension):
     r1 = [
-        dnaio.Sequence("s1", "ACGT", "HHHH"),
-        dnaio.Sequence("s2", "CGCA", "8383"),
+        dnaio.SequenceRecord("s1", "ACGT", "HHHH"),
+        dnaio.SequenceRecord("s2", "CGCA", "8383"),
     ]
     r2 = [
-        dnaio.Sequence("t1", "TCGT", "5HHH"),
-        dnaio.Sequence("t2", "TGCA", "5383"),
+        dnaio.SequenceRecord("t1", "TCGT", "5HHH"),
+        dnaio.SequenceRecord("t2", "TGCA", "5383"),
     ]
-    path = str(tmpdir / ("out.interleaved." + fileformat + extension))
+    path = tmp_path / ("out.interleaved." + fileformat + extension)
 
     with dnaio.open(path, interleaved=True, fileformat=fileformat, mode="w") as f:
         f.write(r1[0], r2[0])
@@ -217,10 +231,10 @@ def test_write_interleaved(tmpdir, fileformat, extension):
         assert formatted_sequences(expected, fileformat) == f.read()
 
 
-def test_append(tmpdir, fileformat, extension):
-    s1 = dnaio.Sequence("s1", "ACGT", "HHHH")
-    s2 = dnaio.Sequence("s2", "CGCA", "8383")
-    path = str(tmpdir / ("out." + fileformat + extension))
+def test_append(tmp_path, fileformat, extension):
+    s1 = dnaio.SequenceRecord("s1", "ACGT", "HHHH")
+    s2 = dnaio.SequenceRecord("s2", "CGCA", "8383")
+    path = tmp_path / ("out." + fileformat + extension)
     with dnaio.open(path, mode="w") as f:
         f.write(s1)
     with dnaio.open(path, mode="a") as f:
@@ -229,17 +243,17 @@ def test_append(tmpdir, fileformat, extension):
         assert formatted_sequences([s1, s2], fileformat) == f.read()
 
 
-def test_append_paired(tmpdir, fileformat, extension):
+def test_append_paired(tmp_path, fileformat, extension):
     r1 = [
-        dnaio.Sequence("s1", "ACGT", "HHHH"),
-        dnaio.Sequence("s2", "CGCA", "8383"),
+        dnaio.SequenceRecord("s1", "ACGT", "HHHH"),
+        dnaio.SequenceRecord("s2", "CGCA", "8383"),
     ]
     r2 = [
-        dnaio.Sequence("t1", "TCGT", "5HHH"),
-        dnaio.Sequence("t2", "TGCA", "5383"),
+        dnaio.SequenceRecord("t1", "TCGT", "5HHH"),
+        dnaio.SequenceRecord("t2", "TGCA", "5383"),
     ]
-    path1 = str(tmpdir / ("out.1." + fileformat + extension))
-    path2 = str(tmpdir / ("out.2." + fileformat + extension))
+    path1 = tmp_path / ("out.1." + fileformat + extension)
+    path2 = tmp_path / ("out.2." + fileformat + extension)
 
     with dnaio.open(path1, file2=path2, fileformat=fileformat, mode="w") as f:
         f.write(r1[0], r2[0])
@@ -251,16 +265,16 @@ def test_append_paired(tmpdir, fileformat, extension):
         assert formatted_sequences(r2, fileformat) == f.read()
 
 
-def test_append_interleaved(tmpdir, fileformat, extension):
+def test_append_interleaved(tmp_path, fileformat, extension):
     r1 = [
-        dnaio.Sequence("s1", "ACGT", "HHHH"),
-        dnaio.Sequence("s2", "CGCA", "8383"),
+        dnaio.SequenceRecord("s1", "ACGT", "HHHH"),
+        dnaio.SequenceRecord("s2", "CGCA", "8383"),
     ]
     r2 = [
-        dnaio.Sequence("t1", "TCGT", "5HHH"),
-        dnaio.Sequence("t2", "TGCA", "5383"),
+        dnaio.SequenceRecord("t1", "TCGT", "5HHH"),
+        dnaio.SequenceRecord("t2", "TGCA", "5383"),
     ]
-    path = str(tmpdir / ("out.interleaved." + fileformat + extension))
+    path = tmp_path / ("out.interleaved." + fileformat + extension)
 
     with dnaio.open(path, interleaved=True, fileformat=fileformat, mode="w") as f:
         f.write(r1[0], r2[0])
@@ -273,6 +287,7 @@ def test_append_interleaved(tmpdir, fileformat, extension):
 
 def make_random_fasta(path, n_records):
     from random import choice
+
     with xopen(path, "w") as f:
         for i in range(n_records):
             name = "sequence_{}".format(i)
@@ -280,8 +295,8 @@ def make_random_fasta(path, n_records):
             print(">", name, "\n", sequence, sep="", file=f)
 
 
-def test_islice_gzip_does_not_fail(tmpdir):
-    path = str(tmpdir / "file.fasta.gz")
+def test_islice_gzip_does_not_fail(tmp_path):
+    path = tmp_path / "file.fasta.gz"
     make_random_fasta(path, 100)
     f = dnaio.open(path)
     next(iter(f))
